import os

from pyftpdlib.filesystems import AbstractedFS

from django.conf import settings


class CustomFileSystem(AbstractedFS):
    def __init__(self, root, cmd_channel):
        root_path = settings.TMP_UPLOADS_DIR
        if not os.path.exists(root_path):
            os.mkdir(root_path)
        root_path = os.path.join(root_path, cmd_channel.user_home)
        if not os.path.exists(root_path):
            os.mkdir(root_path)
        super(CustomFileSystem, self).__init__(root_path, cmd_channel)



import io
from json import JSONDecodeError

import magic
import requests
from django.conf import settings
from django.core.files.uploadedfile import SimpleUploadedFile
from django.db import models

from bloxby.managers import UserBridgeManager
from bloxby.functions import Bloxby
from bloxby.utils import replace_links, extract_zip

bloxby = Bloxby()


class UserBridge(models.Model):
    """You can create this at post_save of PurchasedPackage or something similar, a model that represents that the user
    has purchased a package, and delete it also at post_delete"""
    user = models.OneToOneField(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    bloxby_id = models.IntegerField(blank=True, null=True)
    autologin_token = models.CharField(max_length=20, blank=True, null=True)
    active = models.BooleanField(default=True)
    created = models.DateTimeField(auto_now_add=True)

    objects = UserBridgeManager()

    def __str__(self):
        return f'{self.user.email} - {self.bloxby_id}'

    class Site:
        def __init__(self, sites_id, users_id, sites_name, sitethumb, sites_lastupdate_on):
            self.sites_id = sites_id
            self.users_id = users_id
            self.sites_name = sites_name
            self.sitethumb = sitethumb
            self.sites_lastupdate_on = sites_lastupdate_on

    @property
    def dashboard_url(self):
        if self.active:
            url = settings.BLOXBY_BUILDER['url']
            token = self.autologin_token
            hash_string = bloxby.generate_login_hash()
            if not token:
                token = self.create(self.user).autologin_token
            return f"{url}/auth/alogin/?token={token}&hash={hash_string}"
        return '#'

    def delete(self, using=None, keep_parents=False):
        bloxby.Users.update(self.bloxby_id, status='Inactive')
        self.active = False
        self.save()

    @classmethod
    def create_remote(cls, user, package_id, user_bridge):
        data, success = bloxby.Users.create(
            user.first_name, user.last_name, user.email, bloxby.generate_random_password(), package_id
        )
        if success:
            user_bridge.bloxby_id = data['data']['user']['id']
            user_bridge.autologin_token = data['data']['user']['auto_login_token']
            user_bridge.save()
        return data, success

    @classmethod
    def create(cls, user, package_id=None):
        if not package_id:
            package_id = settings.BLOXBY_BUILDER['default_package_id']
        """
        Package_id is the bloxby package id not the internal one
        Simply call UserBridge.create(package_id, user);
        """
        # Check if user already exists in bloxby
        existings = cls.objects.everything().filter(user=user)
        if existings.exists():
            obj = existings.first()
            obj.active = True
            obj.save()
            new = obj
            exists_remote = True
            if obj.bloxby_id:
                data, is_success = bloxby.Users.update(obj.bloxby_id, status='Active')
                if 'does not belong to a valid user' in str(data):
                    exists_remote = False
            else:
                exists_remote = False
            if not exists_remote:
                cls.create_remote(user, package_id, obj)
        else:
            # Check if email already exists there
            new = cls.objects.create(user=user, override=True)
            data, success = cls.create_remote(user, package_id, new)
            if not success:
                if 'email already used' in str(data):
                    # Hack to find the user again. Not efficient but we have to work with what we have
                    bloxby_users, _ = bloxby.Users.all()
                    for bu in bloxby_users:
                        if user.email in bu['email']:
                            new.bloxby_id = bu['id']
                            new.autologin_token = bu['auto_login_token']
                            new.save()
                            break
        return new

    def user_templates(self):
        base_url = settings.BLOXBY_BUILDER.get('custom_api_url', 'http://159.65.79.47:3000')
        response = requests.get(f'{base_url}/{self.autologin_token}/templates')
        sites = []
        try:
            json_data = response.json()
        except JSONDecodeError:
            json_data = []
        if json_data and isinstance(json_data, list):
            for temp in response.json():
                sites.append(
                    self.Site(
                        temp['sites_id'],
                        temp['users_id'],
                        temp['sites_name'],
                        f'{settings.BLOXBY_BUILDER["url"]}/{temp["sitethumb"]}',
                        temp['sites_lastupdate_on']
                    )
                )
        return sites

    def save_site_from_remote(self, site_id, target, obj_id=None):
        Template.objects.filter(owner=self.user, obj_id=obj_id, target=target, remote_id=site_id).delete()
        template = Template.objects.create(owner=self.user, obj_id=obj_id, target=target, remote_id=site_id)
        base_url = settings.BLOXBY_BUILDER.get('custom_api_url', 'http://159.65.79.47:3000')
        response = requests.get(f'{base_url}/{site_id}/export')
        site_archive = io.BytesIO(response.content)
        unzipped = extract_zip(site_archive)
        for key, file in unzipped.items():
            content_type = magic.from_buffer(file, True)
            file = SimpleUploadedFile(key.split('/')[-1], file, content_type)
            if key.endswith('.html'):
                Page.objects.create(
                    template=template, name=key, html=file, is_built=False
                )
            else:
                TemplateAsset.objects.create(template=template, initial_path=key, file=file)


class Template(models.Model):
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE)
    preview = models.FileField(upload_to='sc', null=True, blank=True)
    target = models.CharField(max_length=100)
    obj_id = models.IntegerField(null=True, blank=True)
    remote_id = models.IntegerField(null=True, blank=True, help_text='ID of the template on the remote server.')

    @property
    def index_page(self):
        index_page = self.page_set.filter(name__endswith='index.html')
        if index_page.exists():
            return index_page.first()
        return None


class Page(models.Model):
    name = models.CharField(max_length=500, null=True, blank=True)
    html = models.FileField(upload_to='template-pages', null=True, blank=True, max_length=500)
    template = models.ForeignKey(Template, on_delete=models.CASCADE, null=True, blank=True)
    is_built = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return self.name

    def process(self):
        replace_links(self)
        self.is_built = True
        self.save()

    def render(self):
        if not self.is_built:
            # The processing should only execute once for each page, and that is at first request
            self.process()
        return self.html.read()

    def absolute_url(self, base_path='/'):
        return f'{base_path}?page={self.name}'


class TemplateAsset(models.Model):
    template = models.ForeignKey(Template, on_delete=models.CASCADE)
    initial_path = models.CharField(max_length=300, help_text='This is the initial path in the html tag')
    file = models.FileField(upload_to='template-assets', max_length=500)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f'{self.initial_path} - {self.file.url}'



import hashlib
import random
import string

from django.conf import settings

from .users import User
from .packages import Package

BASE_URL = settings.BLOXBY_BUILDER['url']
USERNAME = settings.BLOXBY_BUILDER['username']
PASSWORD = settings.BLOXBY_BUILDER['password']
PACKAGE_PREFIX = settings.BLOXBY_BUILDER['package_prefix']
ACCOUNT_EMAIL_PREFIX = settings.BLOXBY_BUILDER['account_email_prefix']
PUBLIC_KEY = settings.BLOXBY_BUILDER['public_key']
AUTOLOGIN_HASH = settings.BLOXBY_BUILDER['autologin_hash']


class Bloxby:
    Users = User(BASE_URL, USERNAME, PASSWORD, PACKAGE_PREFIX, PUBLIC_KEY,ACCOUNT_EMAIL_PREFIX)
    Packages = Package(BASE_URL, USERNAME, PASSWORD, PACKAGE_PREFIX, PUBLIC_KEY, ACCOUNT_EMAIL_PREFIX)

    @staticmethod
    def generate_login_hash():
        return hashlib.md5(f'{BASE_URL}/{AUTOLOGIN_HASH}'.encode('utf-8')).hexdigest()

    @staticmethod
    def generate_random_password():
        letters = string.ascii_lowercase
        return ''.join(random.choice(letters) for i in range(15))



from django.http import HttpResponse, Http404, JsonResponse
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import TemplateView

from bloxby.models import UserBridge, Template, Page, TemplateAsset


class ReceiveFTPItemsView(View):
    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(ReceiveFTPItemsView, self).dispatch(request, *args, **kwargs)

    def on_file_receipt(self, data, template):
        """
        This function is called in the POST, this could be overridden in the class inheriting this class in whatever
        application calling it
        """

    def post(self, request, *args, **kwargs):
        # Handle individual file here, we replace everyone that takes the same target and obj_id
        target = request.POST.get('target')
        obj_id = request.POST.get('obj_id')
        if obj_id == 'null':
            obj_id = None
        files = request.FILES
        user_bloxby_id = request.POST.get('user_bloxby_id')
        user = UserBridge.objects.filter(bloxby_id=user_bloxby_id).first().user
        template, _ = Template.objects.get_or_create(owner=user, obj_id=obj_id, target=target)
        for key, file in files.items():
            if file.name.endswith('.html'):
                page, _ = Page.objects.update_or_create(
                    template=template, name=key, defaults={'html': file, 'is_built': False}
                )
            else:
                asset, _ = TemplateAsset.objects.update_or_create(template=template, initial_path=key,
                                                                  defaults={'file': file})
        self.on_file_receipt(request.POST, template)
        return HttpResponse('success')


class AuthFTPUserView(View):
    @csrf_exempt
    def dispatch(self, request, *args, **kwargs):
        return super(AuthFTPUserView, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        username = request.POST.get('username')
        password = request.POST.get('password')
        try:
            user = UserBridge.objects.get(bloxby_id=username).user
            if user.check_password(password):
                return HttpResponse('Success')
        except UserBridge.DoesNotExist:
            return HttpResponse('Error', status=400)
        return HttpResponse('Error', status=400)


class PageRenderView(View):
    def get(self, request, *args, **kwargs):
        template = Template.objects.first()
        if template:
            page = self.request.GET.get('page')
            if not page:
                html = template.index_page.render()
            else:
                try:
                    html = template.page_set.get(name=page).render()
                except Page.DoesNotExist:
                    raise Http404
        else:
            html = 'Does not exist'
        return HttpResponse(html)


class TestIndexView(TemplateView):
    template_name = 'index.html'

    def get_context_data(self, **kwargs):
        ctx = super(TestIndexView, self).get_context_data(**kwargs)
        return ctx


class ImportSiteView(View):
    def get(self, request, *args, **kwargs):
        """
        Make sure you pass 'sites_id', 'target' and 'obj_id' in URL parameters
        :sites_id: This is the ID of the site you want to import over
        :target: This can be anything, just for you to be able to identify where you are using the template currently,
        I'm using them as either 'event' or 'home'.
        :obj_id: This is going to be the pk of the instance or event as the case may be
        """
        sites_id = request.GET.get('sites_id')
        target = request.GET.get('target', None)
        obj_id = request.GET.get('obj_id', None)
        if not sites_id:
            raise Http404
        # Sample, we will use it for event number 1
        request.user.userbridge.save_site_from_remote(sites_id, target, obj_id)
        return JsonResponse({'status': 'success'})



from django.contrib import admin

from .models import UserBridge, Template, TemplateAsset, Page

admin.site.register(UserBridge)
admin.site.register(Template)
admin.site.register(TemplateAsset)
admin.site.register(Page)



from django import template
from django.conf import settings

from ..models import UserBridge

register = template.Library()


@register.simple_tag(takes_context=True)
def user_builder_dashboard(context):
    user = context['request'].user
    try:
        if user.is_authenticated:
            userbridge = UserBridge.objects.get(user=user)
            if userbridge.bloxby_id:
                return user.userbridge.dashboard_url
    except UserBridge.DoesNotExist:
        # Never mind
        pass
    return UserBridge.create(user).dashboard_url


@register.simple_tag
def builder_create_site():
    return f"{settings.BLOXBY_BUILDER['url']}/sites/create"



# Generated by Django 3.0.5 on 2020-05-06 19:10

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Application',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50)),
                ('auth_url', models.URLField(help_text='URL that authenticated uploader', max_length=500)),
                ('receiving_url', models.URLField(help_text='URL that receives the file', max_length=500)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
            ],
        ),
    ]



import os

import requests
from django.conf import settings
from pyftpdlib.handlers import FTPHandler


class Handler(FTPHandler):
    def send_archive(self, archive_file):
        """Execute the API request to the collector here"""
        print(f'Sent {archive_file} to server.')

    def on_connect(self):
        pass

    def on_disconnect(self):
        pass

    def on_login(self, username):
        pass

    def on_logout(self, username):
        # do something when user logs out
        print('logged out')
        pass

    def on_file_sent(self, file):
        # do something when a file has been sent
        pass

    def on_file_received(self, file):
        pre_path = os.path.join(settings.TMP_UPLOADS_DIR, self.user_home)
        application = self.application
        rel_file_path = file.replace(pre_path, '').lstrip('/')
        with open(file, 'rb') as f:
            resp = requests.post(application.receiving_url, {
                'target': self.target, 'obj_id': self.obj_id, 'user_bloxby_id': self.user
            }, files={
                rel_file_path: (rel_file_path, f.read())
            })
            if 199 < resp.status_code < 300:
                os.remove(file)

    def on_incomplete_file_sent(self, file):
        # do something when a file is partially sent
        pass

    def on_incomplete_file_received(self, file):
        # remove partially uploaded files
        print('Inc ' + file)



# Generated by Django 3.0.6 on 2020-05-07 14:59

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('bloxby', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('preview', models.FileField(blank=True, null=True, upload_to='sc')),
                ('target', models.CharField(max_length=100)),
                ('obj_id', models.IntegerField(blank=True, null=True)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='TemplateAsset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('initial_path', models.CharField(help_text='This is the initial path in the html tag', max_length=300)),
                ('file', models.FileField(max_length=500, upload_to='template-assets')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('template', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bloxby.Template')),
            ],
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=100, null=True)),
                ('html', models.FileField(blank=True, max_length=500, null=True, upload_to='template-pages')),
                ('is_built', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('template', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='bloxby.Template')),
            ],
        ),
    ]
